/* main.c

   Copyright (C) 2002 Joseph Battaglia

   This file is part of phmap

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include "main.h"

int activestat = 0;
int color = 0;
int timeout = 28;
int verbose = 0;
char *darg;
char *farg;
char *oarg;


int main(int argc, char *argv[])
{
	char *clist, *response, *output;
	extern char *optarg;
	int ch;
	int raa = 0, rab = 0, rac = 0;
	signal(SIGKILL, sighandle);
	signal(SIGINT, sighandle);
	while((ch = getopt(argc, argv, "acd:f:ho:t:v")) != -1)
	{
		switch(ch)
		{
			case 'a':
				activestat = 1;
				break;
			case 'c':
				color = 1;
				break;
			case 'd':
				darg = optarg;
				raa = 1;
				break;
			case 'f':
				farg = optarg;
				rab = 1;
				break;
			case 'h':
				printhelp(argv[0]);
				exit(0);
				break;
			case 'o':
				oarg = optarg;
				rac = 1;
				break;
			case 't':
				if((timeout = atoi(optarg)) < 1)
					quit("Timeout too small", 1, 0);
				break;
			case 'v':
				verbose = 1;
				break;
			default:
				printhelp(argv[0]);
				exit(1);
				break;
		}
	}
	if((raa + rab + rac) != 3)
	{
		printhelp(argv[0]);
		quit(NULL, 1, 0);
	}
	printversion();
	openfile(farg, 1);
	openfile(oarg, 2);
	readlist();
	output = malloc(1);
	while((clist = readnext()) != NULL)
	{
		if(verbose)
			printf("\n");
		openmodem(darg);
		dialnumber(clist);
		response = readmodem();
		free(output);
		output = malloc(strlen(clist) + strlen(response) + 4);
		strcpy(output, "");
		strcat(output, clist);
		strcat(output, ": ");
		strcat(output, response);
		strcat(output, "\n");
		writelist(output);
		closemodem();
		if(verbose)
			printf("\n");
	}
	quit("Done", 0, 0);
	return(0);
}

void sighandle(int sig)
{
	if(sig == SIGKILL)
		quit("Caught SIGKILL", 0, 0);
	if(sig == SIGINT)
		quit("Caught SIGINT", 0, 0);
}

void quit(char *msg, int retval, int printerror)
{
	printf("\n");
	if(printerror)
	{
		perror(msg);
	}
	else if(((!retval && verbose) || retval) && msg != NULL)
		printf("%s\n", msg);
	if(color)
	{
		printf("\033[0m");
		fflush(stdout);
	}
	closestreams();
	closemodem();
	exit(retval);
}
