/* modem.c

   Copyright (C) 2002 Joseph Battaglia

   This file is part of phmap

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include "main.h"

extern int activestat;
extern int color;
extern int timeout;
extern int verbose;
int mfd = -1;

void closemodem(void)
{
	if(verbose)
	{
		printf("Closing modem...");
		fflush(stdout);
	}
	if(mfd != -1)
		close(mfd);
	mfd = -1;
	if(verbose)
		printf("Done.\n");
}

void dialnumber(char *number)
{
	char *dialstr;
	char mds[] = "ATDT";
	dialstr = malloc(strlen(number) + strlen(mds) + 1);
	strcat(dialstr, mds);
	strcat(dialstr, number);
	strcat(dialstr, "\r");
	if(verbose)
	{
		if(color)
			printf("\033[36m");
		printf("Dial string: %s\n", dialstr);
		printf("Sending dial string...");
		fflush(stdout);
	}
	if(!verbose)
	{
		if(color)
			printf("\033[36m");
		if(activestat)
			printf("Dialing %s", number);
		else
			printf("Dialing %s: ", number);
		fflush(stdout);
	}
	if(mfd != -1)
		if(write(mfd, dialstr, strlen(dialstr)) == -1)
			quit("Error writing to modem", 1, 1);
	if(verbose)
		printf("Done.\n");
}

void hangup(void)
{
	char mhs[] = "+++ATH0\r";
	if(verbose)
	{
		printf("Hanging up modem...");
		fflush(stdout);
	}
	if(mfd != -1)
		if(write(mfd, mhs, strlen(mhs)) == -1)
			quit("Error writing to modem", 1, 1);
}

void openmodem(char *device)
{
	int flags=0;
	if(verbose)
	{
		printf("Opening modem %s...", device);
		fflush(stdout);
	}
	if((mfd = open(device, O_RDWR)) == -1)
		quit("Error opening modem", 1, 1);
	if(fcntl(mfd, F_GETFL, flags) == -1)
		quit("Error getting flags", 1, 1);
	flags |= O_NONBLOCK;
	if(fcntl(mfd, F_SETFL, flags) == -1)
		quit("Error setting flags", 1, 1);
	if(verbose)
		printf("Done.\n");
}

char *readmodem(void)
{
	char buf[BUFSIZE];
	char tbuf[BUFSIZE * timeout];
	int count = 0;
	int readb = 0;
	bzero(buf, BUFSIZE);
	bzero(tbuf, BUFSIZE * timeout);
	if(verbose)
	{
		if(activestat)
			printf("Waiting for a response");
		else
			printf("Waiting for a response: ");
		fflush(stdout);
	}
	while(count < timeout)
	{
		readb = read(mfd, buf, BUFSIZE);
		strcat(tbuf, buf);
		if(strstr(tbuf, "NO CARRIER") != NULL)
		{
			if(readb == BUFSIZE)
				while(read(mfd, buf, BUFSIZE) != 0);
			if(activestat)
				printf("\n");
			if(color)
				printf("\033[31m");
			printf("NO CARRIER");
			if(color)
				printf("\033[0m");
			printf("\n");
			return("NO CARRIER");
		}
		if(strstr(tbuf, "CONNECT") != NULL)
		{
			if(readb == BUFSIZE)
				while(read(mfd, buf, BUFSIZE) != 0);
			if(activestat)
				printf("\n");
			if(color)
				printf("\033[1m\033[32m");
			printf("CONNECT");
			if(color)
				printf("\033[0m");
			printf("\n");
			return("CONNECT");
		}
		if(strstr(tbuf, "BUSY") != NULL)
		{
			if(readb == BUFSIZE)
				while(read(mfd, buf, BUFSIZE) != 0);
			if(activestat)
				printf("\n");
			if(color)
				printf("\033[33m");
			printf("BUSY");
			if(color)
				printf("\033[0m");
			printf("\n");
			return("BUSY");
		}
		if(strstr(tbuf, "NO DIALTONE") != NULL)
		{
			if(readb == BUFSIZE)
				while(read(mfd, buf, BUFSIZE) != 0);
			if(activestat)
				printf("\n");
			if(color)
				printf("\033[34m");
			printf("NO DIALTONE");
			if(color)
				printf("\033[0m");
			printf("\n");
			return("NO DIALTONE");
		}
		if(count == timeout - 3)
			hangup();
		count++;
		bzero(buf, BUFSIZE);
		if(activestat)
			printf(".");
		fflush(stdout);
		sleep(1);
	}
	if(readb == BUFSIZE)
		while(read(mfd, buf, BUFSIZE) != 0);
			if(activestat)
				printf("\n");
			if(color)
				printf("\033[31m");
			printf("ERROR");
			if(color)
				printf("\033[0m");
			printf("\n");
	return("ERROR");
}
